import sys
from epochis.date_parser import DateParser
from epochis.from_epoch import *


def _print_usage():
    print("Usage")
    print("\tepochis {date}{unit}")
    print("Example")
    print("\tepochis 604m")
    print("Supported Units")
    print("\tm:  months since epoch")
    print("\td:  days since epoch")
    print("\ts:  seconds since epoch")
    print("\tms: milliseconds since epoch")


def _check_args(args):
    if len(args) != 2:
        print("Incorrect number of arguments\n", file=sys.stderr)
        _print_usage()
        sys.exit(2)
    elif args[1] == "--help" or args[1] == "-h":
        _print_usage()
        sys.exit(0)


def main(args):
    _check_args(args)

    try:
        # get date from input
        date_input = args[1]
        date_parser = DateParser(date_input)
        epoch_offset = date_parser.epoch_date()
    except ValueError as ve:
        print(str(ve) + "\n", file=sys.stderr)
        _print_usage()
        sys.exit(2)

    # convert from epoch offset to human readable date
    if epoch_offset.unit == 'm':
        print(month(epoch_offset.offset).strftime("%Y-%m"))
    elif epoch_offset.unit == 'd':
        print(day(epoch_offset.offset))
    elif epoch_offset.unit == 's':
        print(seconds(epoch_offset.offset))
    elif epoch_offset.unit == 'ms':
        print(millis(epoch_offset.offset))
    else:
        print("Unit '{}' is not supported\n".format(epoch_offset.unit), file=sys.stderr)
        _print_usage()
        sys.exit(2)


if __name__ == "__main__":
    main(sys.argv)



"""Methods to convert date offsets from epoch to proper dates
"""
from datetime import *


EPOCH = datetime(1970, 1, 1)
MONTHS_IN_YEAR = 12


def month(months_since_epoch):
    years = int(months_since_epoch / MONTHS_IN_YEAR)
    months = months_since_epoch % MONTHS_IN_YEAR

    return date(EPOCH.year + years, EPOCH.month + months, EPOCH.day)


def day(days_since_epoch):
    return (EPOCH + timedelta(days=days_since_epoch)).date()


def seconds(seconds_since_epoch):
    return EPOCH + timedelta(seconds=seconds_since_epoch)


def millis(millis_since_epoch):
    return EPOCH + timedelta(milliseconds=millis_since_epoch)



import unittest
import sys
import io
from epochis.__main__ import main


def _capture_stdout(func):
    stdout = sys.stdout
    try:
        func_out = io.StringIO()
        sys.stdout = func_out

        func()
        return func_out.getvalue().rstrip()
    finally:
        sys.stdout = stdout


class TestMain(unittest.TestCase):
    def test_months(self):
        out = _capture_stdout(lambda: main(["foo", "604m"]))
        self.assertEqual("2020-05", out)

    def test_days(self):
        out = _capture_stdout(lambda: main(["foo", "18412d"]))
        self.assertEqual("2020-05-30", out)

    def test_seconds(self):
        out = _capture_stdout(lambda: main(["foo", "1590792100s"]))
        self.assertEqual("2020-05-29 22:41:40", out)

    def test_milliseconds(self):
        out = _capture_stdout(lambda: main(["foo", "1590792100123ms"]))
        self.assertEqual("2020-05-29 22:41:40.123000", out)


if __name__ == '__main__':
    unittest.main()



from enum import Enum


class DateLexer:
    """
    Converts the date input into tokens
    """
    def __init__(self, date_input):
        self._input = date_input
        self._look_ahead_index = -1
        self._look_ahead = None

        self._consume()

    def _consume(self):
        self._look_ahead_index += 1
        if len(self._input) > self._look_ahead_index:
            self._look_ahead = self._input[self._look_ahead_index]
        else:
            self._look_ahead = DateTokenType.EOF

    def next(self):
        """
        :return: The next token in the input. If no more input is available an EOF token is returned.
        """
        if self._look_ahead is DateTokenType.EOF:
            return DateToken(DateTokenType.EOF)

        if self._look_ahead.isdigit():
            token = DateToken(DateTokenType.NUMBER, self._look_ahead)
        elif self._look_ahead.isalpha():
            token = DateToken(DateTokenType.LETTER, self._look_ahead)
        else:
            raise ValueError(
                "Unexpected character while parsing date input. Index: {}, Value: '{}'".format(self._look_ahead_index,
                                                                                               self._look_ahead))

        self._consume()
        return token


class DateParser:
    """
    Parses a date input in the format {date}{unit} (e.g. 604m)
    """
    def __init__(self, date_input):
        """
        :param date_input: The date input to parse
        """
        self._lexer = DateLexer(date_input)
        self._look_ahead = self._lexer.next()

    def epoch_date(self):
        """
        Parses the date input
        :return: The parsed input as an epoch offset
        """
        offset = self._offset()
        unit = self._unit()
        self._match(DateTokenType.EOF)

        return EpochOffset(offset, unit)

    def _match(self, expected_token_type):
        if self._look_ahead.type is not expected_token_type:
            raise ValueError(
                "Error parsing date input. Expected token type: {}, but found: {}".format(expected_token_type,
                                                                                          self._look_ahead.type))

    def _consume(self):
        self._look_ahead = self._lexer.next()

    def _consume_until(self, token):
        self._match(token)
        values = [self._look_ahead.value]
        self._consume()

        while self._look_ahead.type is token:
            values.append(self._look_ahead.value)
            self._consume()

        return values

    def _offset(self):
        digits = self._consume_until(DateTokenType.NUMBER)
        return int(''.join(digits))

    def _unit(self):
        letters = self._consume_until(DateTokenType.LETTER)
        return ''.join(letters)


class EpochOffset:
    """
    The date represented as an epoch offset
    """
    def __init__(self, offset, unit):
        """
        :param offset: The number offset from epoch
        :param unit: The offset unit (e.g. 'm' for months)
        """
        self.offset = offset
        self.unit = unit


class DateTokenType(Enum):
    """
    The valid token types for the data lexer
    """
    NUMBER = 1
    LETTER = 2
    EOF = 3


class DateToken:
    """
    The date lexer token
    """
    def __init__(self, token_type, value=None):
        """
        :param token_type: The date lexer token type
        :param value: The token value
        """
        self.type = token_type
        self.value = value



"""Setup for epochis CLI

References:
https://packaging.python.org/tutorials/packaging-projects/
https://chriswarrick.com/blog/2014/09/15/python-apps-the-right-way-entry_points-and-scripts/
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="epochis",
    version="1.1.0",
    author="Rob Reeves",
    author_email="",
    description="A CLI to convert offsets from epoch to human readable dates",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/robreeves/epochis",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'epochis = epochis.__main__:main'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
